import { LoggerFactory } from "./LoggerFactory";
import { LogLevel, LoggerType } from "./Logger";
import { AbstractLogger } from "./LoggerImpl";
/**
 * Defines several date enums used for formatting a date.
 */
export declare enum DateFormatEnum {
    /**
     * Displays as: year-month-day hour:minute:second,millis -> 1999-02-12 23:59:59,123
     * Note the date separator can be set separately.
     */
    Default = 0,
    /**
     * Displays as: year-month-day hour:minute:second -> 1999-02-12 23:59:59
     * Note the date separator can be set separately.
     */
    YearMonthDayTime = 1,
    /**
     * Displays as: year-day-month hour:minute:second,millis -> 1999-12-02 23:59:59,123
     * Note the date separator can be set separately.
     */
    YearDayMonthWithFullTime = 2,
    /**
     * Displays as: year-day-month hour:minute:second -> 1999-12-02 23:59:59
     * Note the date separator can be set separately.
     */
    YearDayMonthTime = 3,
}
/**
 * DateFormat class, stores data on how to format a date.
 */
export declare class DateFormat {
    private _formatEnum;
    private _dateSeparator;
    /**
     * Constructor, can be called empty as it uses defaults.
     * @param formatEnum DateFormatEnum
     * @param dateSeparator Separator used between dates
     */
    constructor(formatEnum?: DateFormatEnum, dateSeparator?: string);
    formatEnum: DateFormatEnum;
    dateSeparator: string;
}
/**
 * Information about the log format, what will a log line look like?
 */
export declare class LogFormat {
    private _dateFormat;
    private _showTimeStamp;
    private _showLoggerName;
    /**
     * Constructor to create a LogFormat. Can be created without parameters where it will use sane defaults.
     * @param dateFormat DateFormat (what needs the date look like in the log line)
     * @param showTimeStamp Show date timestamp at all?
     * @param showLoggerName Show the logger name?
     */
    constructor(dateFormat?: DateFormat, showTimeStamp?: boolean, showLoggerName?: boolean);
    dateFormat: DateFormat;
    showTimeStamp: boolean;
    showLoggerName: boolean;
}
/**
 * Defines a LogGroupRule, this allows you to either have everything configured the same way
 * or for example loggers that start with name model. It allows you to group loggers together
 * to have a certain loglevel and other settings. You can configure this when creating the
 * LoggerFactory (which accepts multiple LogGroupRules).
 */
export declare class LogGroupRule {
    private _regExp;
    private _level;
    private _loggerType;
    private _logFormat;
    private _callBackLogger;
    /**
     * Create a LogGroupRule. Basically you define what logger name(s) match for this group, what level should be used what logger type (where to log)
     * and what format to write in. If the loggerType is custom, then the callBackLogger must be supplied as callback function to return a custom logger.
     * @param regExp Regular expression, what matches for your logger names for this group
     * @param level LogLevel
     * @param logFormat LogFormat
     * @param loggerType Type of logger, if Custom, make sure to implement callBackLogger and pass in, this will be called so you can return your own logger.
     * @param callBackLogger Callback function to return a new clean custom logger (yours!)
     */
    constructor(regExp: RegExp, level: LogLevel, logFormat?: LogFormat, loggerType?: LoggerType, callBackLogger?: (name: string, logGroupRule: LogGroupRule) => AbstractLogger);
    regExp: RegExp;
    level: LogLevel;
    loggerType: LoggerType;
    logFormat: LogFormat;
    callBackLogger: (name: string, logGroupRule: LogGroupRule) => AbstractLogger;
}
/**
 * Options object you can use to configure the LoggerFactory you create at LFService.
 */
export declare class LoggerFactoryOptions {
    private _logGroupRules;
    private _enabled;
    /**
     * Add LogGroupRule, see {LogGroupRule) for details
     * @param rule Rule to add
     * @returns {LoggerFactoryOptions} returns itself
     */
    addLogGroupRule(rule: LogGroupRule): LoggerFactoryOptions;
    /**
     * Enable or disable logging completely for the LoggerFactory.
     * @param enabled True for enabled (default)
     * @returns {LoggerFactoryOptions} returns itself
     */
    setEnabled(enabled: boolean): LoggerFactoryOptions;
    logGroupRules: LogGroupRule[];
    enabled: boolean;
}
/**
 * Create and configure your LoggerFactory from here.
 */
export declare class LFService {
    private static _loggerFactories;
    /**
     * Create a new LoggerFactory with given options (if any). If no options
     * are specified, the LoggerFactory, will accept any named logger and will
     * log on info level by default for, to the console.
     * @param options Options, optional.
     * @returns {LoggerFactory}
     */
    static createLoggerFactory(options?: LoggerFactoryOptions): LoggerFactory;
    /**
     * Closes all Loggers for LoggerFactories that were created.
     * After this call, all previously fetched Loggers (from their
     * factories) are unusable. The factories remain as they were.
     */
    static closeLoggers(): void;
    private static createDefaultOptions();
}
