"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Logger_1 = require("./Logger");
var LoggerFactoryService_1 = require("./LoggerFactoryService");
var ST = require("stacktrace-js");
/**
 * Abstract base logger, extend to easily implement a custom logger that
 * logs wherever you want. You only need to implement doLog(msg: string) and
 * log that somewhere (it will contain format and everything else).
 */
var AbstractLogger = (function () {
    function AbstractLogger(name, rule) {
        this.open = true;
        this.name = name;
        this.rule = rule;
        this.level = rule.level;
    }
    AbstractLogger.prototype.trace = function (msg, error) {
        this.log(Logger_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debug = function (msg, error) {
        this.log(Logger_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.info = function (msg, error) {
        this.log(Logger_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warn = function (msg, error) {
        this.log(Logger_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.error = function (msg, error) {
        this.log(Logger_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatal = function (msg, error) {
        this.log(Logger_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.tracec = function (msg, error) {
        this.logc(Logger_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debugc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.infoc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warnc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.errorc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatalc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.isTraceEnabled = function () {
        return this.level == Logger_1.LogLevel.Trace;
    };
    AbstractLogger.prototype.isDebugEnabled = function () {
        return this.level <= Logger_1.LogLevel.Debug;
    };
    AbstractLogger.prototype.isInfoEnabled = function () {
        return this.level <= Logger_1.LogLevel.Info;
    };
    AbstractLogger.prototype.isWarnEnabled = function () {
        return this.level <= Logger_1.LogLevel.Warn;
    };
    AbstractLogger.prototype.isErrorEnabled = function () {
        return this.level <= Logger_1.LogLevel.Error;
    };
    AbstractLogger.prototype.isFatalEnabled = function () {
        return this.level <= Logger_1.LogLevel.Fatal;
    };
    AbstractLogger.prototype.getLogLevel = function () {
        return this.level;
    };
    AbstractLogger.prototype.log = function (level, msg, error) {
        if (this.open && this.level <= level) {
            this.createMessage(level, msg, new Date(), error);
        }
    };
    AbstractLogger.prototype.logc = function (level, msg, error) {
        if (this.open && this.level <= level) {
            this.createMessage(level, msg(), new Date(), error !== undefined ? error() : undefined);
        }
    };
    AbstractLogger.prototype.isOpen = function () {
        return this.open;
    };
    AbstractLogger.prototype.close = function () {
        this.open = false;
    };
    AbstractLogger.prototype.createMessage = function (level, msg, date, error) {
        var _this = this;
        var lpad = function (value, chars, padWith) {
            var howMany = chars - value.length;
            if (howMany > 0) {
                var res = '';
                for (var i = 0; i < howMany; i++) {
                    res += padWith;
                }
                res += value;
                return res;
            }
            return value;
        };
        var fullYear = function (date) {
            return lpad(date.getFullYear().toString(), 4, '0');
        };
        var month = function (date) {
            return lpad((date.getMonth() + 1).toString(), 2, '0');
        };
        var day = function (date) {
            return lpad(date.getDate().toString(), 2, '0');
        };
        var hours = function (date) {
            return lpad(date.getHours().toString(), 2, '0');
        };
        var minutes = function (date) {
            return lpad(date.getMinutes().toString(), 2, '0');
        };
        var seconds = function (date) {
            return lpad(date.getSeconds().toString(), 2, '0');
        };
        var millis = function (date) {
            return date.getMilliseconds().toString();
        };
        var result = "";
        if (this.rule.logFormat.showTimeStamp) {
            var dateSeparator = this.rule.logFormat.dateFormat.dateSeparator;
            var ds = '';
            switch (this.rule.logFormat.dateFormat.formatEnum) {
                case LoggerFactoryService_1.DateFormatEnum.Default:
                    // yyyy-mm-dd hh:mm:ss,m
                    ds = fullYear(date) + dateSeparator + month(date) + dateSeparator + day(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date) + "," + millis(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearMonthDayTime:
                    ds = fullYear(date) + dateSeparator + month(date) + dateSeparator + day(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearDayMonthWithFullTime:
                    ds = fullYear(date) + dateSeparator + day(date) + dateSeparator + month(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date) + "," + millis(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearDayMonthTime:
                    ds = fullYear(date) + dateSeparator + day(date) + dateSeparator + month(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date);
                    break;
                default:
                    throw new Error("Unsupported date format enum: " + this.rule.logFormat.dateFormat.formatEnum);
            }
            result += ds + " ";
        }
        result += Logger_1.LogLevel[level].toUpperCase() + " ";
        if (this.rule.logFormat.showLoggerName) {
            result += "[" + this.name + "]";
        }
        result += ' ' + msg;
        if (error !== undefined) {
            result += '\n' + error.name + ": " + error.message + "\n@";
            ST.fromError(error, { offline: true }).then(function (frames) {
                var stackStr = (frames.map(function (frame) {
                    return frame.toString();
                })).join('\n  ');
                result += '\n' + stackStr;
                _this.doLog(result);
            });
        }
        else {
            this.doLog(result);
        }
    };
    return AbstractLogger;
}());
exports.AbstractLogger = AbstractLogger;
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw exception.
 */
var ConsoleLoggerImpl = (function (_super) {
    __extends(ConsoleLoggerImpl, _super);
    function ConsoleLoggerImpl(name, rule) {
        _super.call(this, name, rule);
    }
    ConsoleLoggerImpl.prototype.doLog = function (msg) {
        if (console !== undefined) {
            console.log(msg);
        }
        else {
            throw new Error("Console is not defined, cannot log msg: " + msg);
        }
    };
    return ConsoleLoggerImpl;
}(AbstractLogger));
exports.ConsoleLoggerImpl = ConsoleLoggerImpl;
/**
 * Logger which buffers all messages, use with care due to possible high memory footprint.
 * Can be convenient in some cases. Call toString() for full output, or cast to this class
 * and call getMessages() to do something with it yourself.
 */
var MessageBufferLoggerImpl = (function (_super) {
    __extends(MessageBufferLoggerImpl, _super);
    function MessageBufferLoggerImpl(name, rule) {
        _super.call(this, name, rule);
        this.messages = [];
    }
    MessageBufferLoggerImpl.prototype.doLog = function (msg) {
        this.messages.push(msg);
    };
    MessageBufferLoggerImpl.prototype.close = function () {
        this.messages = [];
        _super.prototype.close.call(this);
    };
    MessageBufferLoggerImpl.prototype.getMessages = function () {
        return this.messages;
    };
    MessageBufferLoggerImpl.prototype.toString = function () {
        return this.messages.toString();
    };
    return MessageBufferLoggerImpl;
}(AbstractLogger));
exports.MessageBufferLoggerImpl = MessageBufferLoggerImpl;
//# sourceMappingURL=LoggerImpl.js.map